<?php
class Mappress_Meta {
	static $meta_queue = null;

	static function register() {
		// Change events for custom fields - only if geocoding is configured
		if (!empty(Mappress::$options->metaKeys['post'])) {
			add_action('save_post', array(__CLASS__, 'save_post'), 100, 2);
			add_action('updated_post_meta', array(__CLASS__, 'meta_update_post'), 100, 3);
			add_action('added_post_meta', array(__CLASS__, 'meta_update_post'), 100, 3);
			add_action('deleted_post_meta', array(__CLASS__, 'meta_update_post'), 100, 3);

			// Force-update
			add_action('mappress_update_meta', array(__CLASS__, 'save_post'), 10, 2);
		}

		if (!empty(Mappress::$options->metaKeys['user'])) {
			add_action('edit_user_profile_update', array(__CLASS__, 'save_user'), 100, 3);
			add_action('profile_update', array(__CLASS__, 'save_user'), 100, 3);
			add_action('updated_user_meta', array(__CLASS__, 'meta_update_user'), 100, 3);
			add_action('added_user_meta', array(__CLASS__, 'meta_update_user'), 100, 3);
			add_action('deleted_user_meta', array(__CLASS__, 'meta_update_user'), 100, 3);
		}

		// Flush queue on shutdown
		if (!empty(Mappress::$options->metaKeys['post']) || !empty(Mappress::$options->metaKeys['user']))
			add_action('shutdown', array(__CLASS__, 'flush_queue'), 100, 2);
	}

	/**
	* Create a map from custom fields
	*
	*/
	static function create_meta_map($otype, $oid) {
		$meta_keys = Mappress::$options->metaKeys[$otype];

		// Clear the queue
		self::$meta_queue = null;

		// Get any existing meta map
		$maps = self::get_meta_maps($otype, $oid);

		// If update isn't configured, return without changing map
		if ($maps && !Mappress::$options->metaSyncSave)
			return __('Map already exists, skipped', 'mappress-google-maps-for-wordpress');

		// Clear any old errors
		delete_metadata($otype, $oid, 'mappress_error');

		// Get poi from meta fields
		$poi = self::get_meta_poi($otype, $oid);

		// If no pois were found, delete any existing maps and return
		if (empty($poi) || is_wp_error($poi)) {
			if ($maps) {
				foreach($maps as $map)
					Mappress_Map::delete($map->mapid);
			}
			if (is_wp_error($poi)) {
				$msg = $poi->get_error_message();
				$address = $poi->get_error_data();
				return __('No valid address', 'mappress-google-maps-for-wordpress') . ( ($address) ? " [$address] [$msg]" : " [$msg]");
			}
			return __('No valid address [no error details]', 'mappress-google-maps-for-wordpress');
		}

		// Update existing map or create new map
		$map = (!empty($maps)) ? $maps[0] : new Mappress_Map();
		$map->center = null;
		$map->metaKey = true;
		$map->pois = array($poi);
		$map->title = __('Automatic', 'mappress-google-maps-for-wordpress');
		if (isset($meta_keys['zoom']) && !empty($meta_keys['zoom']))
			$map->zoom = get_metadata($otype, $oid, $meta_keys['zoom'], true);
		$map->otype = $otype;
		$map->oid = $oid;
		$map->save();
		return $poi->address;
	}

	/**
	* Flush queue and enqueue a new object
	*
	*/
	static function enqueue($otype, $oid) {
		// If a different object is already queued, flush the queue first
		if (self::$meta_queue && (self::$meta_queue->otype != $otype || self::$meta_queue->oid != $oid))
			self::flush_queue();

		// Queue current object for update
		self::$meta_queue = (object) array('otype' => $otype, 'oid' => $oid);

	}

	static function flush_queue() {
		$queue = self::$meta_queue;
		if ($queue)
			self::create_meta_map($queue->otype, $queue->oid);
	}

	static function get_meta_maps($otype, $oid) {
		global $wpdb;
		$maps_table = $wpdb->prefix . 'mapp_maps';

		$results = $wpdb->get_results($wpdb->prepare("SELECT mapid FROM $maps_table WHERE otype = %s AND oid = %d", $otype, $oid));

		if ($results === false)
			return false;

		// Return automatic maps (metaKey is set)
		$maps = array();
		foreach($results as $key => $result) {
			$map = Mappress_Map::get($result->mapid);
			if ($map && $map->metaKey)
				$maps[] = $map;
		}
		return $maps;
	}

	static function get_meta_poi($otype, $oid) {
		$meta_keys = Mappress::$options->metaKeys[$otype];
		$poi = new Mappress_Poi();
		$address = array();

		foreach($meta_keys as $key => $meta_field) {

			$meta_value = (empty($meta_field)) ? null : get_metadata($otype, $oid, $meta_field, true);

			if (is_string($meta_value)) {
				$meta_value = trim($meta_value);
				if (empty($meta_value))
					continue;

				if (substr($key, 0, 7) == 'address')
					$address[] = $meta_value;
				else if ($key == 'lat' || $key == 'lng')
					$poi->point[$key] = $meta_value;
				else
					$poi->$key = $meta_value;
			} else if (is_array($meta_value)) {
				// If filter is present it returns the entire POI (used for ACF)
				if (has_filter('mappress_meta_value'))
					return apply_filters('mappress_meta_value', $poi, $meta_value);
				// Any other array isn't valid
				else if (isset($meta_value[0]))
					$address[] = $meta_value[0];
					//return new WP_Error('address', sprintf(__('Field %s is an array'), $key));
			}
		}

		// Concatenate address fields with commas (except ACF, which sends it as a string)
		if (!$poi->address)
			$poi->address = ($address) ? implode(', ', $address) : null;

		// If no address or lat/lng, then skip this post
		if (empty($poi->address) && (empty($poi->point['lat']) || empty($poi->point['lng'])))
			return new WP_Error('address', __('No address or lat/lng specified', 'mappress-google-maps-for-wordpress'));

		// For users, default title to the display name
		if ($otype == 'user' && !$poi->title) {
			$user = get_userdata($oid);
			$poi->title = ($user) ? $user->display_name : '';
		}

		// Geocode the POI
		$result = $poi->geocode();
		if (is_wp_error($result)) {
			$msg = $result->get_error_message();
			add_metadata($otype, $oid, 'mappress_error', $msg = "[$address] [$msg]");
			$result->add_data($poi->address);
			return $result;
		}
		return $poi;
	}

	/**
	* If a map-related custom field has changed, queue the post for update
	* Called by meta_update_post() and meta_update_user()
	*
	*/
	static function meta_update($otype, $oid, $meta_key) {
		$meta_keys = Mappress::$options->metaKeys[$otype];

		// Only configured meta keys
		if (!in_array($meta_key, $meta_keys))
			return;

		// For posts, only process configured post types
		if ($otype == 'post') {
			$post = get_post($oid);
			if (!in_array($post->post_type, Mappress::$options->postTypes))
				return;
		}

		self::enqueue($otype, $oid);
	}

	static function meta_update_post($meta_id, $object_id, $meta_key, $_meta_value = null) {
		self::meta_update('post', $object_id, $meta_key);
	}

	static function meta_update_user($meta_id, $object_id, $meta_key, $_meta_value = null) {
		self::meta_update('user', $object_id, $meta_key);
	}

	/**
	* Create map on post save
	*/
	static function save_post($postid, $post = null) {
		// Ignore revisions
		if (wp_is_post_revision($postid))
			return;

		// Check post exists and is a configured post type
		$post = ($post) ? $post : get_post($postid);
		if (!$post || !in_array($post->post_type, Mappress::$options->postTypes))
			return;

		self::enqueue('post', $postid);
	}

	/**
	* Create map on user save
	*/
	static function save_user($userid) {
		self::enqueue('user', $userid);
	}
}
?>