<?php
class Mappress_Query {
	function __construct() {}

	static function register() {
		// Query parameters
		add_filter('query_vars', array(__CLASS__, 'filter_query_vars'));
		add_filter('parse_query', array(__CLASS__, 'filter_parse_query'));

		// AJAX
		add_action('wp_ajax_mapp_query', array(__CLASS__, 'ajax_query'));
		add_action('wp_ajax_nopriv_mapp_query', array(__CLASS__, 'ajax_query'));
	}

	static function pre_user_query($query) {
		global $wpdb;
		// Only run for 'map' queries
		if ($query->get('map')) {
			$maps_table = $wpdb->prefix . 'mapp_maps';
			$query->query_fields .= ", $maps_table.obj, $maps_table.status ";
			$query->query_from .= " INNER JOIN $maps_table ON $maps_table.oid = $wpdb->users.ID ";
			$query->query_where .= " AND $maps_table.otype = 'user' AND $maps_table.status != 'trash' ";
		}
		return $query;
	}

	/**
	* Parse a query from a shortcode
	*
	* Special 'query' parameter values:
	*   query='' or query='current' : current blog posts (default)
	*   query='all' : show ALL posts with a map
	*
	* @param array $atts - shortcode atts
	* @return array - query for the shortcode (as an array)
	*/
	static function parse_query($atts) {
		global $wp_query;

		$query = (isset($atts['query'])) ? $atts['query'] : array();
		$otype = (isset($atts['otype'])) ? $atts['otype'] : null;

		// Back-compat for old mashup atts ('show' and 'show_query')
		if (isset($atts['show']) && !isset($atts['query'])) {
			$show = $atts['show'];
			$show_query = (isset($atts['show_query'])) ? $atts['show_query'] : '';
			$query = (empty($show) || $show == 'current' || $show == 'all') ? $show : $show_query;
		}

		// Special case 'all'
		if ($query == 'all' || empty($query)) {
			if ($otype == 'user')
				$query = array();
			else
				$query = array('post_type' => 'any');
			return $query;
		}

		// Special case 'current' : set query to pick out the current posts
		if ($query == 'current') {
			if (is_array($wp_query->posts))
				$postids = array_map(function($post) { return (is_int($post)) ? $post : $post->ID; }, $wp_query->posts);
			else
				$postids = array();
			$query = array('post_type' => 'any', 'post__in' => $postids);
			return $query;
		}

		// If query is a querystring, convert it to an array
		if (!is_array($query)) {
			// If query came from a shortcode WP replaces "&, <, >" with html entities, so convert back
			$query = str_replace(array('&amp;', '&#038;', '&lt;', '&gt;'), array('&', '&', '<', '>'), $query);

			// Replace curly braces with brackets for array parameters, e.g. date_query, tax_query, etc.
			$query = str_replace(array('{', '}'), array('[', ']'), $query);

			parse_str($query, $query);

			// Explode array parameters into arrays.  Note that WP *requires* some parameters as comma-separated strings
			$array_keys = array('category__in', 'category__not_in', 'category__and', 'post__in', 'post__not_in', 'tag__in', 'tag__not_in', 'tag__and', 'tag_slug__in', 'tag_slug__and',
				'role__in', 'role__not_in'
			);

			// These parameters may be a string or an array
			$string_array_keys = array('post_type', 'post_status');

			foreach($query as $index => $arg) {
				if (in_array($index, $array_keys))
					$query[$index] = explode(',', $arg);

				else if (in_array($index, $string_array_keys) && strpos($arg, ',') !== false)
					$query[$index] = explode(',', $arg);
			}
		}

		// Substitute query variable strings: e.g. @cat
		foreach($query as $key => $value) {
			if (is_string($value) && substr($value, 0, 1) == '@') {
				$var = substr($value, 1);
				if (isset($wp_query->query_vars[$var]))
					$query[$key] =  $wp_query->query_vars[$var];
				elseif (isset($_REQUEST[$var]))
					$query[$key] = $_REQUEST[$var];
				else
					unset($query[$key]);
			}
		}

		return $query;
	}

	static function ajax_query() {
		ob_start();
		Mappress::ajax_response('OK', self::query($_GET));
	}

	static function query($args = array()) {
		// Query defaults
		$args = (object) wp_parse_args($args, array(
			'debug' => null,
			'filters' => null,
			'otype' => 'post',			// post | user
			'query' => [],
			'url' => null				// For custom development
		));

		self::timing('start');

		// Convert query booleans
		$args->query = Mappress::string_to_boolean($args->query);

		if ($args->otype == 'user')
			$result = self::user_query($args);
		else
			$result = self::post_query($args);

		// Post-query filtering
		$map = apply_filters('mappress_post_query', $result->map, $args);

		// Prepare the map
		$map->prepare();
		self::timing('prepare');

		return (object) array(
			'timing' => ($args->debug) ? self::timing() : null,
			'wpq' => $result->request,
			'pois' => $map->pois
		);
	}

	static function post_query($args) {
		global $wpdb;

		// Query defaults
		$query = wp_parse_args($args->query, array(
			'post_type' => 'any',
		));

		// Query overrides
		$query = array_merge($query, array(
			'fields' => 'ids',
			'ignore_sticky_posts' => true,
			'map' => true,
			'posts_per_page' => -1
		));

		// Filters
		self::parse_filters($query, $args->filters);

		// Allow query to be modified
		$query = apply_filters('mappress_pre_query', $query, $args);

		// Create map
		$map = new Mappress_Map(array('query' => $query, 'otype' => 'post'));

		// Run dummy query, sets up request but bypasses having WP read every post at once (and run out of memory)
		self::toggle_filters(true);
		$wpq = new WP_Query($query);
		self::timing('query');

		// Get POIs
		$map->pois = self::get_post_pois($wpq);

		// Get additional POIs (e.g. from ACF)
		self::toggle_filters(false);
		$map->pois = apply_filters('mappress_query_pois', $map->pois, $query);

		self::timing('get_pois');
		return (object) array('map' => $map, 'request' => $wpq->request);
	}

	/**
	* Get pois for post query
	*/
	static function get_post_pois($wpq) {
		global $wpdb;

		// Rerun the 'real' query to get the map data
		$posts = $wpdb->get_results($wpq->request);

		$pois = array();

		foreach($posts as $post) {
			$mapdata = json_decode($post->obj);

			// Use post object for WP get_permalink()
			$post = new WP_post($post);

			// Saves 300ms.  Note that all fields used in permalinks must be in select statement.
			$post->filter = 'raw';

			foreach ($mapdata->pois as $poidata) {

				$poi = new Mappress_Poi($poidata);

				// No KML in mashups
				if (!Mappress::$options->mashupKml && $poi->type == 'kml')
					continue;

				self::merge_postdata($poi, $post);
				$pois[] = $poi;
			}
		}
		return $pois;
	}

	/**
	* Merge postdata for a POI.  Used by queries and by ACF addon.
	*/
	static function merge_postdata(&$poi, $post) {
		// Permalink (200ms)
		$poi->otype = 'post';
		$poi->oid = $post->ID;
		$poi->url = get_permalink($post); 	// WPML: $poi->url = apply_filters( 'wpml_permalink', home_url() . '?p=' . $_post->ID);

		// Post data: title/excerpt/thumbnail (thumbnail could be inner joined to speed up)
		if (Mappress::$options->mashupBody == 'post') {
			$poi->title = $post->post_title;
			$poi->body = $poi->get_post_excerpt($post);
		}

		// Thumbnails (only if they'll be displayed)
		if (Mappress::$options->thumbsPopup || Mappress::$options->thumbsList) {

			// Thumbs can come from poi or from post thumbnail, depending on setting mashupThumbs
			if (Mappress::$options->mashupThumbs == 'poi') {
				$poi->update_images();
			} else {
				$thumbnail_id = (int) get_post_meta( $post->ID, '_thumbnail_id', true );
				if ($thumbnail_id)
					$poi->images = array( (object) array('id' => $thumbnail_id, 'type' => 'image'));
			}
		}
	}

	static function user_query($args) {
		// Query overrides
		$query = array_merge($args->query, array(
			'fields' => array('obj'),
			'map' => true,
		));

		// Filters
		self::parse_filters($query, $args->filters);

		// Join to maps table
		add_action( 'pre_user_query', array(__CLASS__, 'pre_user_query'));

		$wpq = new WP_User_Query($query);
		$map = new Mappress_Map(array('query' => $query, 'otype' => 'user'));

		foreach($wpq->results as $result) {
			$mapdata = json_decode($result->obj);

			if ($mapdata->pois) {
				foreach($mapdata->pois as $poidata) {
					$poi = new Mappress_Poi($poidata);
					$poi->oid = $mapdata->oid;
					$map->pois[] = $poi;
				}
			}
		}

		self::timing('get_pois');
		return (object) array('map' => $map, 'request' => $wpq->request);
	}

	static function parse_filters(&$query, $filters) {
		if (empty($filters) || !is_array($filters))
			return;

		// Expect array of filters [ filter_key => values ], where values may be a single value or an array
		foreach($filters as $key => $value) {

			if (empty($value))
				continue;

			// Some query parts require an array
			$values = (is_array($value)) ? $value : [$value];

			switch($key) {
				case 'post' :
					$query['post_type'] = $values;
					break;

				case 'text' :
					$query['s'] = $value;
					break;

				case 'role' :
					$query['role__in'] = $values;
					break;

				default :
					$query['tax_query'][] = array('taxonomy' => $key, 'field' => 'slug', 'terms' => $values);

					// Try to remove any querystring parameters for same key
					if ($key == 'post_tag')
						unset($query['tag'], $query['tag_id'], $query['tag__and'], $query['tag__in'], $query['tag__not_in'], $query['tag_slug__and'], $query['tag_slug__in']);
					else if ($key == 'category')
						unset($query['cat'], $query['category_name'], $query['category__and'], $query['category__in'], $query['category__not_in']);
					break;
			}
		}
	}

	/**
	* Check if a query is empty, i.e. returns no POIs
	* Note: user queries not implemented yet
	*
	*/
	static function is_empty($query) {
		global $wpdb;

		// Query overrides
		$query = array_merge($query, array(
			'cache_results' => false,
			'fields' => 'ids',
			'ignore_sticky_posts' => true,
			'map' => true,
			'posts_per_page' => -1
		));

		// Query defaults
		$query = wp_parse_args($query, array(
			'otype' => 'post',			// post | user
		));

		// Run dummy query, then 'real' query, and check that at least 1 post will be returned
		self::toggle_filters(true);
		$wpq = new WP_Query($query);
		self::toggle_filters(false);
		$posts = $wpdb->get_results($wpq->request);

		return (count($posts) == 0);
	}

	/**
	* Remove the is_admin flag from map queries
	* During frontend AJAX calls WP thinks it's running in the admin, so WP_Query will return private, draft, etc. posts that should be hidden
	* See: http://core.trac.wordpress.org/ticket/12400
	*
	* @param mixed $query
	*/
	static function filter_parse_query($query) {
		if (isset($query->query_vars['map']) && $query->query_vars['map'])
			$query->is_admin = false;
		return $query;
	}

	/**
	* Add map query var
	*/
	static function filter_query_vars ( $qv ) {
		$qv[] = 'map';
		return $qv;
	}

	/**
	* Replace query fields with custom fields
	*/
	static function posts_fields($fields, $query) {
		global $wpdb;
		if ($query->get('map')) {
			$maps_table = $wpdb->prefix . 'mapp_maps';
			$fields = (Mappress::$options->mashupBody == 'post') ? "$wpdb->posts.*" : "$wpdb->posts.ID, $wpdb->posts.post_name, $wpdb->posts.post_date, $wpdb->posts.post_type";
			$fields .= ", $maps_table.obj";
		}
		return $fields;
	}

	/**
	* Add map ID to groupby
	*/
	static function posts_groupby($groupby, $query) {
		global $wpdb;
		if ($query->get('map')) {
			$maps_table = $wpdb->prefix . 'mapp_maps';
			$groupby = ($groupby) ? "$groupby, $maps_table.mapid" : "$maps_table.mapid";
		}
		return $groupby;
	}

	/**
	* Join map tables into query
	*/
	static function posts_join_paged($join, $query) {
		global $wpdb;
		if ($query->get('map')) {
			$maps_table = $wpdb->prefix . 'mapp_maps';
			$join .= " INNER JOIN $maps_table ON ($maps_table.oid = $wpdb->posts.ID AND $maps_table.otype = 'post' AND $maps_table.status != 'trashed') ";
		}
		return $join;
	}

	/**
	*  Return a single post ID to short-circuit WP reading from the DB
	*
	*/
	static function posts_pre_query($posts, $query) {
		// Only run for 'map' queries
		if ($query->get('map'))
			return array(1);
	}

	static function timing($name = null) {
		static $times, $last_time;

		if ($name)
			$times[$name] = microtime(true);

		else {
			$s = '';
			$total = 0;
			foreach($times as $name => $time) {
				if ($name != 'start') {
					$d = round(1000 * ($time - $last_time), 0);
					$s .= " $name : " . $d;
					$total = $total + $d;
				}
				$last_time = $time;
			}
			return "$s total: $total";
		}
	}

	static function toggle_filters($toggle) {
		// Join mapdata
		// For WP 6.0 these were moved from posts_join_request, post_fields_request, and posts_groupby_request
		// Line 2993 in wp-includes/class-wp-query was stripping out all of the clauses:
		// $clauses = (array) apply_filters_ref_array( 'posts_clauses_request', array( $clauses, &$this ) );
		if ($toggle) {
			add_filter('posts_join_paged', array(__CLASS__, 'posts_join_paged'), 10, 2);
			add_filter('posts_fields', array(__CLASS__, 'posts_fields'), 10, 2);
			add_filter('posts_groupby', array(__CLASS__, 'posts_groupby'), 10, 2);
			add_filter('posts_pre_query', array(__CLASS__, 'posts_pre_query'), 10, 2);
		} else {
			remove_filter('posts_join_paged', array(__CLASS__, 'posts_join_paged'), 10);
			remove_filter('posts_fields', array(__CLASS__, 'posts_fields'), 10);
			remove_filter('posts_groupby', array(__CLASS__, 'posts_groupby'), 10);
			remove_filter('posts_pre_query', array(__CLASS__, 'posts_pre_query'), 10);
		}
	}
}
?>